@@ -21,8 +21,10 @@ from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as lib_constants
 
+from neutron.agent.l3 import agent as l3_agent
 from neutron.agent.l3 import namespace_manager
 from neutron.agent.l3 import namespaces
+from neutron.agent.l3 import router_info
 from neutron.agent.linux import ip_lib
 from neutron.common import utils
 from neutron.tests.common import machine_fixtures
@@ -67,7 +69,7 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
             mock.call('router', 'after_create', self.agent, router=router),
             mock.call('router', 'before_update', self.agent, router=router),
             mock.call('router', 'after_update', self.agent, router=router),
-            mock.call('router', 'before_delete', self.agent, router=router),
+            mock.call('router', 'before_delete', self.agent, payload=mock.ANY),
             mock.call('router', 'after_delete', self.agent, router=router)]
         event_handler.assert_has_calls(expected_calls)
 
@@ -107,39 +109,53 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
 
         self.assertIsNone(device.route.get_gateway())
 
+    def test_router_processing_pool_size(self):
+        mock.patch.object(router_info.RouterInfo, 'initialize').start()
+        mock.patch.object(router_info.RouterInfo, 'process').start()
+        self.agent.l3_ext_manager = mock.Mock()
+        mock.patch.object(router_info.RouterInfo, 'delete').start()
+        mock.patch.object(registry, 'notify').start()
+
+        router_info_1 = self.generate_router_info(False)
+        r1 = self.manage_router(self.agent, router_info_1)
+        self.assertEqual(l3_agent.ROUTER_PROCESS_GREENLET_MIN,
+                         self.agent._pool.size)
+
+        router_info_2 = self.generate_router_info(False)
+        r2 = self.manage_router(self.agent, router_info_2)
+        self.assertEqual(l3_agent.ROUTER_PROCESS_GREENLET_MIN,
+                         self.agent._pool.size)
+
+        router_info_list = [r1, r2]
+        for _i in range(l3_agent.ROUTER_PROCESS_GREENLET_MAX + 1):
+            ri = self.generate_router_info(False)
+            rtr = self.manage_router(self.agent, ri)
+            router_info_list.append(rtr)
+
+        self.assertEqual(l3_agent.ROUTER_PROCESS_GREENLET_MAX,
+                         self.agent._pool.size)
+
+        for router in router_info_list:
+            self.agent._safe_router_removed(router.router_id)
+
+        agent_router_info_len = len(self.agent.router_info)
+        if agent_router_info_len < l3_agent.ROUTER_PROCESS_GREENLET_MIN:
+            self.assertEqual(l3_agent.ROUTER_PROCESS_GREENLET_MIN,
+                             self.agent._pool.size)
+        elif (l3_agent.ROUTER_PROCESS_GREENLET_MIN <= agent_router_info_len <=
+                l3_agent.ROUTER_PROCESS_GREENLET_MAX):
+            self.assertEqual(agent_router_info_len,
+                             self.agent._pool.size)
+        else:
+            self.assertEqual(l3_agent.ROUTER_PROCESS_GREENLET_MAX,
+                             self.agent._pool.size)
+
     def _make_bridge(self):
         bridge = framework.get_ovs_bridge(utils.get_rand_name())
         bridge.create()
         self.addCleanup(bridge.destroy)
         return bridge
 
-    def test_external_network_bridge_change(self):
-        bridge1, bridge2 = self._make_bridge(), self._make_bridge()
-        self.agent.conf.set_override('external_network_bridge',
-                                     bridge1.br_name)
-        router_info = self.generate_router_info(False)
-        router = self.manage_router(self.agent, router_info)
-        gw_port = router.router['gw_port']
-        gw_inf_name = router.get_external_device_name(gw_port['id'])
-
-        self.assertIn(gw_inf_name,
-                      [v.port_name for v in bridge1.get_vif_ports()])
-        # changeing the external_network_bridge should have no impact since
-        # the interface exists.
-        self.agent.conf.set_override('external_network_bridge',
-                                     bridge2.br_name)
-        self.manage_router(self.agent, router_info)
-        self.assertIn(gw_inf_name,
-                      [v.port_name for v in bridge1.get_vif_ports()])
-        self.assertNotIn(gw_inf_name,
-                         [v.port_name for v in bridge2.get_vif_ports()])
-        namespaces.Namespace.delete(router.router_namespace)
-        self.manage_router(self.agent, router_info)
-        self.assertIn(gw_inf_name,
-                      [v.port_name for v in bridge2.get_vif_ports()])
-        self.assertNotIn(gw_inf_name,
-                         [v.port_name for v in bridge1.get_vif_ports()])
-
     def test_legacy_router_ns_rebuild(self):
         router_info = self.generate_router_info(False)
         router = self.manage_router(self.agent, router_info)
@@ -196,8 +212,6 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
         # Mock the plugin RPC API so a known external network id is returned
         # when the router updates are processed by the agent
         external_network_id = framework._uuid()
-        self.mock_plugin_api.get_external_network_id.return_value = (
-            external_network_id)
 
         # Plug external_gateway_info in the routers that are not going to be
         # deleted by the agent when it processes the updates. Otherwise,
@@ -377,10 +391,10 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
                       fixed_ip_address_scope='scope2')
         router.process()
 
-        br_ex = framework.get_ovs_bridge(
-            self.agent.conf.external_network_bridge)
+        br_int = framework.get_ovs_bridge(
+            self.agent.conf.ovs_integration_bridge)
         src_machine = self.useFixture(
-            machine_fixtures.FakeMachine(br_ex, '19.4.4.12/24'))
+            machine_fixtures.FakeMachine(br_int, '19.4.4.12/24'))
         # Floating ip should work no matter of address scope
         net_helpers.assert_ping(src_machine.namespace, fip_same_scope)
         net_helpers.assert_ping(src_machine.namespace, fip_diff_scope)
@@ -391,11 +405,11 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
 
         gw_port = router.get_ex_gw_port()
         gw_ip = self._port_first_ip_cidr(gw_port).partition('/')[0]
-        br_ex = framework.get_ovs_bridge(
-            self.agent.conf.external_network_bridge)
+        br_int = framework.get_ovs_bridge(
+            self.agent.conf.ovs_integration_bridge)
 
         src_machine = self.useFixture(
-            machine_fixtures.FakeMachine(br_ex, '19.4.4.12/24', gw_ip))
+            machine_fixtures.FakeMachine(br_int, '19.4.4.12/24', gw_ip))
         # For the internal networks that are in the same address scope as
         # external network, they can directly route to external network
         net_helpers.assert_ping(src_machine.namespace, machine_same_scope.ip)
