@@ -15,7 +15,9 @@
 #    under the License.
 
 import mock
+from neutron_lib import constants as p_const
 
+from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge_test_base
 
@@ -38,7 +40,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=23)),
+                table_id=23),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -46,7 +49,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 ],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -57,19 +61,34 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 ],
                 match=ofpp.OFPMatch(),
                 priority=3,
-                table_id=60)),
+                table_id=60),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=24)),
+                table_id=24),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(vlan_vid=4095),
                 priority=65535,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[
+                    ofpp.OFPInstructionActions(
+                        ofp.OFPIT_APPLY_ACTIONS, [
+                            ofpp.OFPActionOutput(ofp.OFPP_NORMAL, 0)
+                        ]),
+                ],
+                match=ofpp.OFPMatch(),
+                priority=3,
+                table_id=61),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -94,7 +113,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=port,
                     vlan_vid=segmentation_id | ofp.OFPVID_PRESENT),
                 priority=3,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -120,7 +140,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=port,
                     vlan_vid=ofp.OFPVID_NONE),
                 priority=3,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -174,8 +195,9 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=4,
-                table_id=1)),
+                priority=20,
+                table_id=1),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -187,8 +209,9 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=4,
-                table_id=60)),
+                priority=20,
+                table_id=60),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -203,14 +226,14 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
         expected = [
             call.uninstall_flows(
                 strict=True,
-                priority=4,
+                priority=20,
                 table_id=1,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT)),
             call.uninstall_flows(
                 strict=True,
-                priority=4,
+                priority=20,
                 table_id=60,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -242,8 +265,9 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=4,
-                table_id=2)),
+                priority=20,
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -255,8 +279,9 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=4,
-                table_id=60)),
+                priority=20,
+                table_id=60),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -271,14 +296,14 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
         expected = [
             call.uninstall_flows(
                 strict=True,
-                priority=4,
+                priority=20,
                 table_id=2,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT)),
             call.uninstall_flows(
                 strict=True,
-                priority=4,
+                priority=20,
                 table_id=60,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -301,7 +326,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     eth_src=mac,
                     in_port=port),
                 priority=4,
-                table_id=0))
+                table_id=0),
+                           active_bundle=None)
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -329,7 +355,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     eth_src=mac,
                     in_port=port),
                 priority=2,
-                table_id=0))
+                table_id=0),
+                           active_bundle=None)
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -361,7 +388,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888,
                 ),
                 priority=2,
-                table_id=24)),
+                table_id=24),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -375,7 +403,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888,
                 ),
                 priority=2,
-                table_id=24)),
+                table_id=24),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -388,7 +417,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888,
                 ),
                 priority=10,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -409,7 +439,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888,
                 ),
                 priority=2,
-                table_id=24)),
+                table_id=24),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -421,7 +452,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888
                 ),
                 priority=2,
-                table_id=24)),
+                table_id=24),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -432,7 +464,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     in_port=8888,
                 ),
                 priority=10,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -452,3 +485,36 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
             call.uninstall_flows(table_id=24, in_port=port),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
+
+    def _test_delete_dvr_dst_mac_for_arp(self, network_type):
+        if network_type == p_const.TYPE_VLAN:
+            table_id = constants.DVR_TO_SRC_MAC_VLAN
+        else:
+            table_id = constants.DVR_TO_SRC_MAC
+
+        vlan_tag = 1111
+        gateway_mac = '00:02:b3:13:fe:3e'
+        dvr_mac = '00:02:b3:13:fe:3f'
+        rtr_port = 8888
+        self.br.delete_dvr_dst_mac_for_arp(network_type=network_type,
+                                           vlan_tag=vlan_tag,
+                                           gateway_mac=gateway_mac,
+                                           dvr_mac=dvr_mac,
+                                           rtr_port=rtr_port)
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call.uninstall_flows(
+                strict=True,
+                priority=5,
+                table_id=table_id,
+                match=ofpp.OFPMatch(
+                    eth_dst=dvr_mac,
+                    vlan_vid=vlan_tag | ofp.OFPVID_PRESENT)),
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_delete_dvr_dst_mac_for_arp_vlan(self):
+        self._test_delete_dvr_dst_mac_for_arp(network_type='vlan')
+
+    def test_delete_dvr_dst_mac_for_arp_tunnel(self):
+        self._test_delete_dvr_dst_mac_for_arp(network_type='vxlan')
